#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// CCI updated by ThatManFromTexas January 04,2011, to allow background color changes depending on CCI Threshold
    /// </summary>
    [Description("CCI with BackGround Color Change By ThatManFromTexas")]
    public class CCIwArrows : Indicator
    {
        #region Variables
		private int period2 = 50;
        private int cCIthreshold = 0;
		private int period = 14; // Default setting for 
		private double line1Value = 100; // Default setting for Line1Value
        private double line2Value = -100; // Default setting for Line2Value
        private double line3Value = 200; // Default setting for Line3Value
        private double line4Value = -200; // Default setting for Line4Value
        private bool colorbackground = true;
		private int opacity		= 50;
		private Color bgColorUp = Color.RoyalBlue;
		private Color bgColorDown = Color.DeepPink;
		private bool 	drawarrows	=true;
		private Color arrowsUpColor		= Color.Cyan;
		private Color arrowsDownColor	= Color.Red;
		private DataSeries signal;
		
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {	
            Add(new Plot(new Pen(Color.White, 2), PlotStyle.Line, "CCImiddle"));			// Between margins.
            Add(new Plot(new Pen(Color.DarkOrange, 2), PlotStyle.Line, "CCIoverbought"));// Upper = Overbought (>+100)
            Add(new Plot(new Pen(Color.DarkOrange, 2), PlotStyle.Line, "CCIoversold"));	// Lower = Oversold   (<-100)
			Add(new Plot(new Pen(Color.Lime, 1), PlotStyle.Bar, "CCIbarHi"));			// Histogram above zero.
			Add(new Plot(new Pen(Color.Red, 1), PlotStyle.Bar, "CCIbarLo"));			// Histogram below zero.
			Add(new Plot(Color.Yellow, PlotStyle.Line, "Line1"));
            Add(new Plot(Color.Yellow, PlotStyle.Line, "Line2"));
            Add(new Plot(Color.Yellow, PlotStyle.Line, "Line3"));
            Add(new Plot(Color.Yellow, PlotStyle.Line, "Line4"));
			Add(new Plot(Color.Magenta, PlotStyle.Line, "CCITurbo"));
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			PaintPriceMarkers = false;
			DisplayInDataBox = false;
			DrawOnPricePanel=true;
			
			signal				= new DataSeries(this);
			
			
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	Plots[0].Min = Line2Value;									// Not oversold or overbought.
			Plots[0].Max = Line1Value;
			Plots[1].Min = Line1Value;										// Overbought is above this.
			Plots[2].Max = Line2Value;									// Oversold is below this.
			Plots[3].Min = 0;										// Overbought is above this.
			Plots[4].Max = 0;	
			
			
			double mean = 0;
			for (int idx = Math.Min(CurrentBar, Period2 - 1); idx >= 0; idx--)
				mean += Math.Abs(Typical[idx] - SMA(Typical, Period2)[0]);
			double plotval = (Typical[0] - SMA(Typical, Period2)[0]) / (mean == 0 ? 1 : (0.015 * (mean / Math.Min(Period2, CurrentBar + 1))));for (int idx = Math.Min(CurrentBar, Period2 - 1); idx >= 0; idx--)
			
				
			signal.Set(plotval);
			
			if (colorbackground)
			{
			if (CCI(Period)[0] > CCIthreshold && CCI(Period2)[0]> CCIthreshold )
				BackColorAll = Color.FromArgb(opacity,BgColorUp);
			
			else if (CCI(Period)[0] <= -CCIthreshold && CCI(Period2)[0]<= -CCIthreshold)
				BackColorAll = Color.FromArgb(opacity,BgColorDown);	
			
			else BackColorAll= Color.Empty;
			}
				
			if (Line1Value != 0) Line1.Set(Line1Value);
            if (Line2Value != 0) Line2.Set(Line2Value);
            if (Line3Value != 0) Line3.Set(Line3Value);
            if (Line4Value != 0) Line4.Set(Line4Value);
			
			
			
			
			// Note, some of the following settings will be ignored based on the Min & Max values defined in Initialize() above:
			CCImiddle.Set( plotval );								// Blue for NOT over-sold or over-bought.
			CCIoverbought.Set( plotval );							// DarkGreen for over-bought.
			CCIoversold.Set( plotval );								// DarkRed for over-sold.
			CCIbarHi.Set( plotval );								// Green histogram for over-bought.
			CCIbarLo.Set( plotval );								// Red histogram for over-sold.
			CCITurbo.Set(CCI(Period)[0]);
			
			if(Drawarrows)
			{
				if(signal[0]>CCIthreshold && signal[1]<=CCIthreshold && CCITurbo[0]>CCIthreshold && CCITurbo[1]<=CCIthreshold)
					
				DrawArrowUp("tag1"+ CurrentBar, true, 0, Low[0] - TickSize, ArrowsUpColor);
				
				else if(signal[0]<-CCIthreshold && signal[1]>=-CCIthreshold && CCITurbo[0]<=-CCIthreshold&& CCITurbo[1]>=-CCIthreshold)
				
				DrawArrowDown("tag-1"+ CurrentBar, true, 0, High[0] + TickSize, ArrowsDownColor);
			}	
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCImiddle
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIoverbought
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIoversold
        {
            get { return Values[2]; }
        }

       [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIbarHi
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIbarLo
        {
            get { return Values[4]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCITurbo
        {
            get { return Values[9]; }
        }

		[Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period2
        {
            get { return period2; }
			set { period2 = Math.Max(1, value); }
			
        }
		
		[Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
			set { period = Math.Max(1, value); }
			
        }
		
		
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Line1
        {
            get { return Values[5]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Line2
        {
            get { return Values[6]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Line3
        {
            get { return Values[7]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Line4
        {
            get { return Values[8]; }
        }

        [Description("Line 1 value")]
        [GridCategory("Parameters")]
        public double Line1Value
        {
            get { return line1Value; }
            set { line1Value = value; }
        }

        [Description("Line 2 value")]
        [GridCategory("Parameters")]
        public double Line2Value
        {
            get { return line2Value; }
            set { line2Value = value; }
        }

        [Description("Line 3 value")]
        [GridCategory("Parameters")]
        public double Line3Value
        {
            get { return line3Value; }
            set { line3Value = value; }
        }

        [Description("Line 4 Value")]
        [GridCategory("Parameters")]
        public double Line4Value
        {
            get { return line4Value; }
            set { line4Value = value; }
        }
		
		// BackGround Colors
		
		
		
		[Description("Color Background?")]
		[GridCategory("BackGround")]
		[Gui.Design.DisplayNameAttribute("01. Color Background?")]
		public bool ColorBackground
		{
			get { return colorbackground; }
			set { colorbackground = value; }
		}
		
		[Description("Threshold for BackGround Color Change")]
        [GridCategory("BackGround")]
		[Gui.Design.DisplayNameAttribute("02. CCI Threshold?")]
        public int CCIthreshold
        {
            get { return cCIthreshold; }
			set { cCIthreshold = value; }
			
        }
		
		/// <summary>
        /// </summary>
        [XmlIgnore()]
        [Description("Background opacity 1=lightest, 9=darkest")]
        [GridCategory("BackGround")]
        [Gui.Design.DisplayNameAttribute("03. Opacity")]
        public int Opacity
        {
            get { return opacity; }
            set { opacity =  value; }
        }
		
		[XmlIgnore()]
        [Description("Color Above CCI Threshold")]
        [GridCategory("BackGround")]
        [Gui.Design.DisplayNameAttribute("04. Up BG color")]
        public Color BgColorUp
        {
            get { return bgColorUp; }
            set { bgColorUp = value; }
        }
		
        [Browsable(false)]
        public string bgColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bgColorUp); }
            set { bgColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
        [XmlIgnore()]
        [Description("Color Below CCI Threshold")]
        [GridCategory("BackGround")]
        [Gui.Design.DisplayNameAttribute("05. Down BG color")]
        public Color BgColorDown
        {
            get { return bgColorDown; }
            set { bgColorDown = value; }
        }
		
        [Browsable(false)]
        public string bgColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bgColorDown); }
            set { bgColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		
		[Description("Draw Arrows?")]
		[GridCategory("Colors")]
		[Gui.Design.DisplayNameAttribute(".01 Draw Arrows?")]
		public bool Drawarrows
		{
			get { return drawarrows; }
			set { drawarrows = value; }
		}
		
		[XmlIgnore()]
        [Description("Color for Up Arrow")]
        [GridCategory("Colors")]
		[Gui.Design.DisplayNameAttribute(".02 Color for Up Arrows?")]
        public Color ArrowsUpColor
        {
            get { return arrowsUpColor; }
            set { arrowsUpColor = value; }
        }
		
		[Browsable(false)]
		public string ArrowsUpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(ArrowsUpColor); }
			set { ArrowsUpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for Down Arrows")]
        [GridCategory("Colors")]
		[Gui.Design.DisplayNameAttribute(".03 Color for Down Arrows?")]
        public Color ArrowsDownColor
        {
            get { return arrowsDownColor; }
            set { arrowsDownColor = value; }
        }
		
		[Browsable(false)]
		public string ArrowsDownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(ArrowsDownColor); }
			set { ArrowsDownColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CCIwArrows[] cacheCCIwArrows = null;

        private static CCIwArrows checkCCIwArrows = new CCIwArrows();

        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        public CCIwArrows CCIwArrows(Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            return CCIwArrows(Input, arrowsDownColor, arrowsUpColor, bgColorDown, bgColorUp, cCIthreshold, colorBackground, drawarrows, line1Value, line2Value, line3Value, line4Value, opacity, period, period2);
        }

        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        public CCIwArrows CCIwArrows(Data.IDataSeries input, Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            if (cacheCCIwArrows != null)
                for (int idx = 0; idx < cacheCCIwArrows.Length; idx++)
                    if (cacheCCIwArrows[idx].ArrowsDownColor == arrowsDownColor && cacheCCIwArrows[idx].ArrowsUpColor == arrowsUpColor && cacheCCIwArrows[idx].BgColorDown == bgColorDown && cacheCCIwArrows[idx].BgColorUp == bgColorUp && cacheCCIwArrows[idx].CCIthreshold == cCIthreshold && cacheCCIwArrows[idx].ColorBackground == colorBackground && cacheCCIwArrows[idx].Drawarrows == drawarrows && Math.Abs(cacheCCIwArrows[idx].Line1Value - line1Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line2Value - line2Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line3Value - line3Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line4Value - line4Value) <= double.Epsilon && cacheCCIwArrows[idx].Opacity == opacity && cacheCCIwArrows[idx].Period == period && cacheCCIwArrows[idx].Period2 == period2 && cacheCCIwArrows[idx].EqualsInput(input))
                        return cacheCCIwArrows[idx];

            lock (checkCCIwArrows)
            {
                checkCCIwArrows.ArrowsDownColor = arrowsDownColor;
                arrowsDownColor = checkCCIwArrows.ArrowsDownColor;
                checkCCIwArrows.ArrowsUpColor = arrowsUpColor;
                arrowsUpColor = checkCCIwArrows.ArrowsUpColor;
                checkCCIwArrows.BgColorDown = bgColorDown;
                bgColorDown = checkCCIwArrows.BgColorDown;
                checkCCIwArrows.BgColorUp = bgColorUp;
                bgColorUp = checkCCIwArrows.BgColorUp;
                checkCCIwArrows.CCIthreshold = cCIthreshold;
                cCIthreshold = checkCCIwArrows.CCIthreshold;
                checkCCIwArrows.ColorBackground = colorBackground;
                colorBackground = checkCCIwArrows.ColorBackground;
                checkCCIwArrows.Drawarrows = drawarrows;
                drawarrows = checkCCIwArrows.Drawarrows;
                checkCCIwArrows.Line1Value = line1Value;
                line1Value = checkCCIwArrows.Line1Value;
                checkCCIwArrows.Line2Value = line2Value;
                line2Value = checkCCIwArrows.Line2Value;
                checkCCIwArrows.Line3Value = line3Value;
                line3Value = checkCCIwArrows.Line3Value;
                checkCCIwArrows.Line4Value = line4Value;
                line4Value = checkCCIwArrows.Line4Value;
                checkCCIwArrows.Opacity = opacity;
                opacity = checkCCIwArrows.Opacity;
                checkCCIwArrows.Period = period;
                period = checkCCIwArrows.Period;
                checkCCIwArrows.Period2 = period2;
                period2 = checkCCIwArrows.Period2;

                if (cacheCCIwArrows != null)
                    for (int idx = 0; idx < cacheCCIwArrows.Length; idx++)
                        if (cacheCCIwArrows[idx].ArrowsDownColor == arrowsDownColor && cacheCCIwArrows[idx].ArrowsUpColor == arrowsUpColor && cacheCCIwArrows[idx].BgColorDown == bgColorDown && cacheCCIwArrows[idx].BgColorUp == bgColorUp && cacheCCIwArrows[idx].CCIthreshold == cCIthreshold && cacheCCIwArrows[idx].ColorBackground == colorBackground && cacheCCIwArrows[idx].Drawarrows == drawarrows && Math.Abs(cacheCCIwArrows[idx].Line1Value - line1Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line2Value - line2Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line3Value - line3Value) <= double.Epsilon && Math.Abs(cacheCCIwArrows[idx].Line4Value - line4Value) <= double.Epsilon && cacheCCIwArrows[idx].Opacity == opacity && cacheCCIwArrows[idx].Period == period && cacheCCIwArrows[idx].Period2 == period2 && cacheCCIwArrows[idx].EqualsInput(input))
                            return cacheCCIwArrows[idx];

                CCIwArrows indicator = new CCIwArrows();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ArrowsDownColor = arrowsDownColor;
                indicator.ArrowsUpColor = arrowsUpColor;
                indicator.BgColorDown = bgColorDown;
                indicator.BgColorUp = bgColorUp;
                indicator.CCIthreshold = cCIthreshold;
                indicator.ColorBackground = colorBackground;
                indicator.Drawarrows = drawarrows;
                indicator.Line1Value = line1Value;
                indicator.Line2Value = line2Value;
                indicator.Line3Value = line3Value;
                indicator.Line4Value = line4Value;
                indicator.Opacity = opacity;
                indicator.Period = period;
                indicator.Period2 = period2;
                Indicators.Add(indicator);
                indicator.SetUp();

                CCIwArrows[] tmp = new CCIwArrows[cacheCCIwArrows == null ? 1 : cacheCCIwArrows.Length + 1];
                if (cacheCCIwArrows != null)
                    cacheCCIwArrows.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheCCIwArrows = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CCIwArrows CCIwArrows(Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            return _indicator.CCIwArrows(Input, arrowsDownColor, arrowsUpColor, bgColorDown, bgColorUp, cCIthreshold, colorBackground, drawarrows, line1Value, line2Value, line3Value, line4Value, opacity, period, period2);
        }

        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        public Indicator.CCIwArrows CCIwArrows(Data.IDataSeries input, Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            return _indicator.CCIwArrows(input, arrowsDownColor, arrowsUpColor, bgColorDown, bgColorUp, cCIthreshold, colorBackground, drawarrows, line1Value, line2Value, line3Value, line4Value, opacity, period, period2);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CCIwArrows CCIwArrows(Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            return _indicator.CCIwArrows(Input, arrowsDownColor, arrowsUpColor, bgColorDown, bgColorUp, cCIthreshold, colorBackground, drawarrows, line1Value, line2Value, line3Value, line4Value, opacity, period, period2);
        }

        /// <summary>
        /// CCI with BackGround Color Change By ThatManFromTexas
        /// </summary>
        /// <returns></returns>
        public Indicator.CCIwArrows CCIwArrows(Data.IDataSeries input, Color arrowsDownColor, Color arrowsUpColor, Color bgColorDown, Color bgColorUp, int cCIthreshold, bool colorBackground, bool drawarrows, double line1Value, double line2Value, double line3Value, double line4Value, int opacity, int period, int period2)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CCIwArrows(input, arrowsDownColor, arrowsUpColor, bgColorDown, bgColorUp, cCIthreshold, colorBackground, drawarrows, line1Value, line2Value, line3Value, line4Value, opacity, period, period2);
        }
    }
}
#endregion
